/* 
   Copyright 2018-2024 LEADS-EXTRACTOR.com
   End-user license agreement (EULA 2018-2021)
 */
renderIndex();
var gmRegex = new RegExp(/^https:\/\/www.google([a-z0-9-_\.]*)\.([a-z0-9-_]+)+\/maps\/*/)
  , btnGoogleExport = document.getElementById("btnGoogleExport")
  , btnGoogleClear = document.getElementById("btnGoogleClear")
  , incorrectpageContainer = document.getElementById("incorrect_page")
  , correctpageContainer = document.getElementById("correct_page")
  , MAX_DATA_FREE_VERSION = 10
  , results = null
  , showSettings = false
  , exportOptions = {}
  , exportOptionsDefault = {
    'name': true,
    'fulladdress': true,
    'street': true,
    'locality': true,
    'municipality': true,
    'postal code': true,
    'email': true,
    'phone': true,
    'business categories': true,
    'total reviews': true,
    'average rating': true,
    'latitude': true,
    'longitude': true,
    'website': true,
    'domain': true,
    'cid': true,
    'place_id': true,
    'google maps_url': true,
    'linkedin_url': true,
    'facebook_url': true,
    'twitter_url': true,
    'instagram_url': true,
    'youtube_url': true,
    'pinterest_url': true,
    'facebook pixel': true,
    'google tag manager': true,
    'google analytics': true,
    'opening hours': true,
    'price_level': true,
    'business_status': true,
    'photos': true
  };

chrome.storage.local.get(['bg_businesses_temp', 'exportOptions', 'TR', 'TF'], (storage) => {
  let results = null;
  try {
    results = JSON.parse(storage['bg_businesses_temp']) || null;
  }
  catch(err)
  {
  }
  
  const exportOpt = storage['exportOptions'] || null;
  const TR = storage['TR'] || 0;
  const TF = storage['TF'] || 0;

  if (results != null)
  {
    showLastResults(results, TR, TF);
  }
  if (exportOpt != null) {
    exportOptions = JSON.parse(exportOpt);
  } else {
    exportOptions = exportOptionsDefault;
  }

  createExportOptions();
});

function toggleSettings() {
  if (!showSettings) {
    showSettings = true;
    document.getElementById("export").classList.remove("hidden");
    document.getElementById("exportSettings").innerHTML = "Hide settings";
  } else {
    showSettings = false;
    document.getElementById("export").classList.add("hidden");
    document.getElementById("exportSettings").innerHTML = "Show export settings";
  }
  
}

function resetExportOptions() {
  exportOptions = exportOptionsDefault;
}

async function updateExportOptions(event) {
  const key = event.target.id;
  const newValue = event.target.checked;

  exportOptions[key] = newValue;
  await chrome.storage.local.set({'exportOptions': JSON.stringify(exportOptions)});
}

checkURL();
function checkURL() {
  chrome.tabs.query({'active': true, 'lastFocusedWindow': true}, function (tabs) {
    var currentTabUrl = tabs[0].url;

    if (gmRegex.test(currentTabUrl) || results != null) {
      document.getElementById('correct_page').classList.remove("hidden");
      document.getElementById('export').classList.remove("hidden");
      createExportOptions();
    } else {
      document.getElementById('correct_page').classList.add("hidden");
      document.getElementById('export').classList.add("hidden");
      document.getElementById('incorrect_page').classList.remove("hidden");
    }

    if (tabs[0].incognito) {
      document.getElementById('disable_incognito').classList.remove("hidden");
    }

    document.getElementById("exportSettings").onclick = function() {
      toggleSettings();
    }

  });
}

document.getElementById("cur_version").innerHTML = chrome.runtime.getManifest().version;
getUserIdentity();
async function getUserIdentity() {
  await chrome.identity.getProfileUserInfo(function(userInfo) {
    if (userInfo.email !== "" || userInfo.id !== "")
    {
      user = userInfo;
      chrome.storage.local.set({'LastUserCallData': JSON.stringify(userInfo)});
      chrome.runtime.setUninstallURL("https://www.leads-extractor.com/uninstall-google-maps?email="+user.email);
    } else {
      chrome.runtime.setUninstallURL("https://www.leads-extractor.com/uninstall-google-maps");
    }
  });
}

var interval = setInterval(async function() {
  correctpageContainer.classList.remove("hidden");

  await chrome.storage.local.get(['bg_businesses_temp', 'TF', 'TR'], (storage) => {
    const TF = storage['TF'] || 0;
    const TR = storage['TR'] || 0;

    if (TF != 0 && TF != null) {
      let results = null;
      try {
        results = JSON.parse(storage['bg_businesses_temp']) || null;
      }
      catch(err)
      {
      }

      if (results != null) {
        let nonChecked = results.filter(i => !i.checked);
        nonChecked.forEach(e => {
          var val = extractSocialMedia(e.html, e.url);
          var index = results.findIndex(x => x.place_id === e.place_id);
          let merged = { ...results[index], ...val };
          results[index] = merged;
          chrome.storage.local.set({ 'bg_businesses_temp': JSON.stringify(results) });
        });
      }

      renderProgress(TR, TF);
  
      if (TR >= (TF / 2)) {
        clearInterval(interval);
        btnGoogleClear.classList.remove("hidden");
        btnGoogleExport.classList.remove("hidden");
        document.getElementById("counterResults").innerHTML = "Press the green EXPORT button - It is ready!";
        document.getElementById("last_results").innerHTML = "";
        hideProgress();
      }
    }
  });

}, 1000);

function createExportOptions() {
  document.getElementById("export").innerHTML = "<hr><span style='font-weight:bold'>Export only:</span><br>";
  for (const [key, value] of Object.entries(exportOptions)) {
    var node = document.createElement("input", {type: 'checkbox'});
    node.classList.add("btn-check");
    node.setAttribute("type", 'checkbox');
    node.setAttribute("id", key);
    node.addEventListener('input', updateExportOptions);
    if(value) {
      node.checked = true;
    }
    node.setAttribute("autocomplete", 'off');
    var labelNode = document.createElement('label');
    labelNode.setAttribute("for", key);
    labelNode.classList.add("btn");
    labelNode.classList.add("btn-outline-primary");
    labelNode.innerHTML = key;


    document.getElementById("export").appendChild(node);
    document.getElementById("export").appendChild(labelNode);
  }
  document.getElementById("export").appendChild(document.createElement("hr"));
  document.getElementById("export").classList.add("hidden");
}

function showLastResults(results, TR, TF)
{
    if(results.length > 0) {
      btnGoogleClear.classList.remove("hidden");
      btnGoogleExport.classList.remove("hidden");
    } else {
      return;
    }

    renderProgress(results.length, TF);
    document.getElementById("counterResults").innerHTML = "Total results: " + results.length;
}

btnGoogleClear.onclick = function() {
  document.getElementById("counterResults").innerHTML = "Total results: " + 0;
  chrome.storage.local.set({'TF': 0, 'TR': 0});
  btnGoogleExport.classList.add("hidden");
  clearLocalStorageCollection()
};

async function clearLocalStorageCollection()
{
  hideProgress();
  document.getElementById("last_results").innerHTML = "";
  chrome.storage.local.remove(['bg_businesses_temp']);
  chrome.storage.local.set({'TF': 0, 'TR': 0});
  try {
    await chrome.runtime.sendMessage({ type: 'delete' });
  } catch (error) {
    console.log(error);
  }
}

btnGoogleExport.onclick = function()
{
  hideProgress();

  chrome.storage.local.get(['bg_businesses_temp'], async (storage) => {
    await chrome.storage.local.set({'TF': 0, 'TR': 0});

    const data = JSON.parse(storage['bg_businesses_temp']) || null;

    var max_data = 999999999;

    if (data.length <= 0)
    {
      return;
    }
    
    btnGoogleClear.classList.add("hidden");
    btnGoogleExport.classList.add("hidden");
    document.getElementById("counterResults").innerHTML = "";
  
    "FULL" != _STATUS && (max_data = MAX_DATA_FREE_VERSION);
  
    var limitedDataByLicense = data.slice(0, max_data).map(i => {
      return i;
    });

    var filterDataByPreference = limitedDataByLicense.map(v => {

      for (const [key, value] of Object.entries(v)) {
        if (!exportOptions[key]) {
          delete v[key];
        }
      }

      return v;
    });

    downloadCSV(filterDataByPreference)
    clearLocalStorageCollection();
  });
};

function convertArrayOfObjectsToCSV(args) {  
  var result, ctr, keys, columnDelimiter, lineDelimiter, data;

  data = args.data || null;
  if (data == null || !data.length) {
    return null;
  }

  columnDelimiter = args.columnDelimiter || ',';
  lineDelimiter = args.lineDelimiter || '\n';

  keys = Object.keys(exportOptions);

  result = '';
  result += keys.join(columnDelimiter);
  result += lineDelimiter;

  data.forEach(function(item) {
    ctr = 0;
    keys.forEach(function(key) {
      if (ctr > 0) result += columnDelimiter;
      
      if (item[key] != null || item[key] != undefined) {
        result += item[key];
      } else {
        result += "";
      }
      
      ctr++;
    });
    result += lineDelimiter;
  });

  return result;
}

function downloadCSV(a) {
    var c = convertArrayOfObjectsToCSV({
        data: a
    });
    if (null != c) {
        window.URL = window.URL || window.webkitURL;
        a = "google_maps_leads.csv";
        blob = new Blob(["\ufeff", c], {
          type: "application/octet-binary"
        });
        var b = document.createElement("a");
          b.setAttribute("id", "btnDownload");
          b.setAttribute("class", "btn btn-success");
          b.setAttribute("target", "_self");
          b.setAttribute("href", window.URL.createObjectURL(blob));
          b.setAttribute("download", a);
          document.getElementById("correct_page").appendChild(b);
          document.getElementById("btnDownload").innerHTML = "Download Results (as .CSV)";
    }
}
;document.addEventListener('DOMContentLoaded', function() {
    var links = document.getElementsByTagName("a");
    for (var i = 0; i < links.length; i++) {
        (function() {
            var ln = links[i];
            var location = ln.href;
            ln.onclick = function() {
                chrome.tabs.create({
                    active: true,
                    url: location
                });
            }
            ;
        }
        )();
    }
});
function extractSocialMedia(html, url)
{
  var parser = new DOMParser();
  var website = parser.parseFromString(html, "text/html");
  var linkedin = website.querySelectorAll('a[href*="linkedin"]');
  var fb = website.querySelectorAll('a[href*="facebook"]');
  var twitter = website.querySelectorAll('a[href*="twitter"]');
  var email = website.querySelectorAll('a[href*="mailto"]');
  var instagram = website.querySelectorAll('a[href*="instagram"]');
  var youtube = website.querySelectorAll('a[href*="youtube"]');

  if(linkedin.length > 0) {
    linkedin = linkedin[0].href;
  } else { linkedin = ' ';}

  if(fb.length > 0) {
    fb = fb[0].href;
  } else { fb = ' ';}

  if(twitter.length > 0) {
    twitter = twitter[0].href;
  } else { twitter = ' ';}

  if(instagram.length > 0) {
    instagram = instagram[0].href;
  } else { instagram = ' ';}

  if(youtube.length > 0) {
    youtube = youtube[0].href;
  } else { youtube = ' ';}

  if(email.length > 0) {
    email = (email[0].href).substr(7);
  } else {
    var tempEmail = website.querySelectorAll('a[href*="@'+url+'"]');
    if (tempEmail.length > 0) {
      email = tempEmail[0].href;
    } else {
      email = ' ';
    }
  }

  email = decodeURIComponent(email);
  email = cleanEmailStringFromParameters(email);

  var fbp = checkFbTracking(html);
  var gmtag = checkGTMTracking(html);
  var ga = checkGATracking(html);

  var values = {
    'email': email,
    'facebook_url': fb,
    'linkedin_url': linkedin,
    'twitter_url': twitter,
    'instagram_url': instagram,
    'youtube_url': youtube,
    'facebook pixel': fbp,
    'google tag manager': gmtag,
    'google analytics': ga,
    'checked': true
  };

  return values;
}

function cleanEmailStringFromParameters(string)
{
  var i = string.indexOf("?");

  if (i === -1)
  {
    return string;
  }

  return string.substring(i, 0);
}

function checkFbTracking(html) {
  var val1 = html.indexOf("fbq");
  var val2 = html.indexOf("connect.facebook");

  if (val1 >= 0 || val2 >= 0) {
    return 'Yes';
  } else {
    return 'No'
  }
}
function checkGTMTracking(html) {
  var val1 = html.indexOf("gtag");
  var val2 = html.indexOf("googletagmanager.com/gtm.js");

  if (val1 >= 0 || val2 >= 0) {
    return 'Yes';
  } else {
    return 'No'
  }
}
function checkGATracking(html) {
  var val1 = html.indexOf("_gaq");
  var val2 = html.indexOf("google-analytics.com/analytics.js");

  if (val1 >= 0 || val2 >= 0) {
    return 'Yes';
  } else {
    return 'No'
  }
}